/* ==============================================================================================================================================
 * LIBRARY:         DCF77
 * FILE:            DCF77_settings.h
 * LAST REV:        1.0
 * AUTHOR:          Ivo Colleoni
 * DEVICE:          PIC18F
 * CREATED:         28/09/2016
 * UPDATED:         
 *
 * DESCRIP:         Settaggi per libreria DCF77
 * ============================================================================================================================================== */

/* CONFIGURAZIONE =============================================================================================================================== */
// PIN di connessione del segnale DCF77 (Via interrupt usare INT0~INT2)
#define DCF77_SIGNAL            PORTBbits.RB0
#define DCF77_SIGNAL_RB         0                   // riportare RB di collegamento
#define DCF77_INT_IF            INTCONbits.INT0IF
#define DCF77_INT_IE            INTCONbits.INT0IE
#define DCF77_INT_EDGE          INTCON2bits.INTEDG0                                
//#define DCF77_INT_PRIOROIRY     INTCON2bits.      // impostare se collegato su pin diverso da RB0, RB0  sempre priorit alta.
// PIN di connessione del segnale di abilitazione, se non presente commentare                        
//#define DCF77_PDN                   PORTCbits.RC0     

// Logica del segnale DCF77
#define DCF77_ACTIVE_HIGH
//#define DCF77_ACTIVE_LOW

// Logica del PIN di attivazione
#define DCF77_PDN_ACTIVE_LOW
//#define DCF77_PDN_ACTIVE_HIGH

/* TIMINGS DCF77 in mS */
#define DCF77_SCAN_TIME             1

#define MINUTE_MARK_COUNT           1850      /DCF77_SCAN_TIME                          // tempo in ms in cui il segnale deve restare basso per individuare un minute mark
#define WINDOW_SECOND_LOW           900       /DCF77_SCAN_TIME                          // finestre per avere un impulso al secondo valido
#define WINDOW_SECOND_HIGH          1100      /DCF77_SCAN_TIME
#define WINDOW_CHECK_LOW            130       /DCF77_SCAN_TIME                          // finestra di controllo (il segnale resta alto da 130 a 170ms solo per segnale 1)
#define WINDOW_CHECK_HIGH           170       /DCF77_SCAN_TIME  
#define END_CHECK_COUNT             180       /DCF77_SCAN_TIME                          // tempo per iniziare il conteggio dei campioni 
#define RESTART_COUNT               850       /DCF77_SCAN_TIME                          // tempo per riavviare l'interrupt in attesa di un sengale di secondo valido
#define THRESHOLD                   30        /DCF77_SCAN_TIME                          // numero di campioni che devono essere a 1 per considerare il segnale come 1

/* Impostazioni timer, definire i registri e modificare la funzione di inizializzazione a seconda del PIC/timer usato */
#define DCF77_TIMER_ON              T0CONbits.TMR0ON
#define DCF77_TIMER_INT_IE          INTCONbits.TMR0IE
#define DCF77_TIMER_INT_IF          INTCONbits.TMR0IF
#define DCF77_TIMER_INT_PRIORITY    INTCON2bits.TMR0IP 
#define DCF77_TMR                   TMR0
#if (DCF77_SCAN_TIME == 1)
#define DCF77_TIMER_PRELOAD         61534 // 1 ms
#elif(DCF77_SCAN_TIME == 10)
#define DCF77_TIMER_PRELOAD         25534 // 10 ms
#else 
#error Timing non gestito
#endif


// Preload del timer da calcolare a seconda del clock e delle impostazioni del timer usate per avere interrupt di 1ms

// TIMER 0, Configurazioni per avere 1 interrupt ogni 10ms = (65536-preload)*(4/fosc)*prescaler = (65536-61534)*(4/64000000)*4
void DCF77_set_timer(void){
    T0CONbits.T0PS0 = 1;                                                                // prescaler 1/4
    T0CONbits.T0PS1 = 0;                                                                //
    T0CONbits.T0PS2 = 0;
    T0CONbits.PSA = 0;                                                                  // prescaler assegnato
    T0CONbits.T0CS = 0;                                                                 // da clock interno fosc/4
    T0CONbits.T08BIT = 0;                                                               // 16 BIT
    
    DCF77_TIMER_ON = OFF;                                                               // timer spento
    TMR0 = DCF77_TIMER_PRELOAD;                                                         // preload timer 0

    DCF77_TIMER_INT_PRIORITY = 1;                                                       // priorit alta
    DCF77_TIMER_INT_IE= OFF;                                                            // abilito interrupt timer 0 
}